@@ -68,6 +68,7 @@ typedef struct
 struct engine_gpgsm
 {
   assuan_context_t assuan_ctx;
+  char *version;
 
   int lc_ctype_set;
   int lc_messages_set;
@@ -113,6 +114,10 @@ struct engine_gpgsm
 
   /* Memory data containing diagnostics (--logger-fd) of gpgsm */
   gpgme_data_t diagnostics;
+
+  struct {
+    unsigned int offline : 1;
+  } flags;
 };
 
 typedef struct engine_gpgsm *engine_gpgsm_t;
@@ -122,6 +127,13 @@ static void gpgsm_io_event (void *engine,
                             gpgme_event_io_t type, void *type_data);
 
 
+/* Return true if the engine's version is at least VERSION.  */
+static int
+have_gpgsm_version (engine_gpgsm_t gpgsm, const char *version)
+{
+  return _gpgme_compare_versions (gpgsm->version, version);
+}
+
 
 static char *
 gpgsm_get_version (const char *file_name)
@@ -254,6 +266,9 @@ gpgsm_release (void *engine)
 
   gpgsm_cancel (engine);
 
+  if (gpgsm->version)
+    free (gpgsm->version);
+
   gpgme_data_release (gpgsm->diagnostics);
 
   free (gpgsm->colon.attic.line);
@@ -281,12 +296,20 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
   char *optstr;
   unsigned int connect_flags;
 
-  (void)version; /* Not yet used.  */
-
   gpgsm = calloc (1, sizeof *gpgsm);
   if (!gpgsm)
     return gpg_error_from_syserror ();
 
+  if (version)
+    {
+      gpgsm->version = strdup (version);
+      if (!gpgsm->version)
+	{
+	  err = gpg_error_from_syserror ();
+	  goto leave;
+	}
+    }
+
   gpgsm->status_cb.fd = -1;
   gpgsm->status_cb.dir = 1;
   gpgsm->status_cb.tag = 0;
@@ -549,8 +572,6 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
 	  || _gpgme_io_set_close_notify (gpgsm->output_cb.fd,
 					 close_notify_handler, gpgsm)
 	  || _gpgme_io_set_close_notify (gpgsm->message_cb.fd,
-					 close_notify_handler, gpgsm)
-	  || _gpgme_io_set_close_notify (gpgsm->diag_cb.fd,
 					 close_notify_handler, gpgsm)))
     {
       err = gpg_error (GPG_ERR_GENERAL);
@@ -575,9 +596,9 @@ gpgsm_new (void **engine, const char *file_name, const char *home_dir,
     _gpgme_io_close (gpgsm->output_cb.server_fd);
   if (gpgsm->message_cb.server_fd != -1)
     _gpgme_io_close (gpgsm->message_cb.server_fd);
+#endif
   if (gpgsm->diag_cb.server_fd != -1)
     _gpgme_io_close (gpgsm->diag_cb.server_fd);
-#endif
 
   if (err)
     gpgsm_release (gpgsm);
@@ -603,6 +624,8 @@ gpgsm_set_engine_flags (void *engine, const gpgme_ctx_t ctx)
     }
   else
     *gpgsm->request_origin = 0;
+
+  gpgsm->flags.offline = (ctx->offline && have_gpgsm_version (gpgsm, "2.1.6"));
 }
 
 
@@ -668,6 +691,23 @@ gpgsm_assuan_simple_command (engine_gpgsm_t gpgsm, const char *cmd,
   gpg_error_t err, cb_err;
   char *line;
   size_t linelen;
+  struct io_select_fd_s fds[2];
+  struct io_cb_data iocb_data;
+  int nfds = 0;
+
+  iocb_data.handler_value = gpgsm->diag_cb.data;
+  iocb_data.op_err = 0;
+
+  memset (fds, 0, sizeof (struct io_select_fd_s)*2);
+  fds[nfds].fd = gpgsm->status_cb.fd;
+  fds[nfds].for_read = 1;
+  nfds++;
+  if (gpgsm->diag_cb.fd != -1)
+    {
+      fds[nfds].fd = gpgsm->diag_cb.fd;
+      fds[nfds].for_read = 1;
+      nfds++;
+    }
 
   err = assuan_write_line (ctx, cmd);
   if (err)
@@ -676,6 +716,24 @@ gpgsm_assuan_simple_command (engine_gpgsm_t gpgsm, const char *cmd,
   cb_err = 0;
   do
     {
+      fds[0].signaled = fds[1].signaled = 0;
+      if (gpgsm->status_cb.fd != -1
+	  && _gpgme_io_select (fds, nfds, 0) < 0)
+	{
+          err = gpg_error_from_syserror ();
+	  break;
+	}
+
+      if (gpgsm->diag_cb.fd != -1 && fds[1].signaled)
+	{
+	  err = _gpgme_data_inbound_handler (&iocb_data, gpgsm->diag_cb.fd);
+	  if (err)
+	    break;
+	}
+
+      if (gpgsm->status_cb.fd != -1 && !fds[0].signaled)
+	continue;
+
       err = assuan_read_line (ctx, &line, &linelen);
       if (err)
 	break;
@@ -1129,7 +1187,7 @@ add_io_cb (engine_gpgsm_t gpgsm, iocb_data_t *iocbd, gpgme_io_cb_t handler)
   gpgme_error_t err;
 
   TRACE_BEG  (DEBUG_ENGINE, "engine-gpgsm:add_io_cb", gpgsm,
-              "fd %d, dir %d", iocbd->fd, iocbd->dir);
+              "fd=%d, dir %d", iocbd->fd, iocbd->dir);
   err = (*gpgsm->io_cbs.add) (gpgsm->io_cbs.add_priv,
 			      iocbd->fd, iocbd->dir,
 			      handler, iocbd->data, &iocbd->tag);
@@ -1143,28 +1201,15 @@ add_io_cb (engine_gpgsm_t gpgsm, iocb_data_t *iocbd, gpgme_io_cb_t handler)
 
 
 static gpgme_error_t
-start (engine_gpgsm_t gpgsm, const char *command)
+prepare (engine_gpgsm_t gpgsm)
 {
-  gpgme_error_t err;
+  gpgme_error_t err = 0;
   assuan_fd_t afdlist[5];
+  /* FIXME: the type int and assuan_fd_t may be incompatible.  */
   int fdlist[5];
   int nfds;
   int i;
 
-  if (*gpgsm->request_origin)
-    {
-      char *cmd;
-
-      cmd = _gpgme_strconcat ("OPTION request-origin=",
-                              gpgsm->request_origin, NULL);
-      if (!cmd)
-        return gpg_error_from_syserror ();
-      err = gpgsm_assuan_simple_command (gpgsm, cmd, NULL, NULL);
-      free (cmd);
-      if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
-        return err;
-    }
-
   /* We need to know the fd used by assuan for reads.  We do this by
      using the assumption that the first returned fd from
      assuan_get_active_fds() is always this one.  */
@@ -1188,7 +1233,36 @@ start (engine_gpgsm_t gpgsm, const char *command)
 
   gpgsm->status_cb.fd = _gpgme_io_dup (fdlist[0]);
   if (gpgsm->status_cb.fd < 0)
-    return gpg_error_from_syserror ();
+    err = gpg_error_from_syserror ();
+
+  return err;
+}
+
+
+static gpgme_error_t
+start (engine_gpgsm_t gpgsm, const char *command)
+{
+  gpgme_error_t err;
+
+  if (*gpgsm->request_origin)
+    {
+      char *cmd;
+
+      cmd = _gpgme_strconcat ("OPTION request-origin=",
+                              gpgsm->request_origin, NULL);
+      if (!cmd)
+        return gpg_error_from_syserror ();
+      err = gpgsm_assuan_simple_command (gpgsm, cmd, NULL, NULL);
+      free (cmd);
+      if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
+        return err;
+    }
+
+  gpgsm_assuan_simple_command (gpgsm,
+                               gpgsm->flags.offline ?
+                               "OPTION offline=1":
+                               "OPTION offline=0" ,
+                               NULL, NULL);
 
   if (_gpgme_io_set_close_notify (gpgsm->status_cb.fd,
 				  close_notify_handler, gpgsm))
@@ -1226,7 +1300,7 @@ gpgsm_reset (void *engine)
 
   /* IF we have an active connection we must send a reset because we
      need to reset the list of signers.  Note that RESET does not
-     reset OPTION commands. */
+     reset all OPTION commands. */
   return (gpgsm->assuan_ctx
           ? gpgsm_assuan_simple_command (gpgsm, "RESET", NULL, NULL)
           : 0);
@@ -1234,6 +1308,38 @@ gpgsm_reset (void *engine)
 #endif
 
 
+/* Send the input-size-hint option.  Note that we need to send it
+ * always so that we don't actually use a wrong hint from the last
+ * command.  */
+static gpgme_error_t
+send_input_size_hint (engine_gpgsm_t gpgsm, gpgme_data_t data)
+{
+  gpg_error_t err;
+  uint64_t value;
+  char numbuf[50];  /* Large enough for even 2^128 in base-10.  */
+  char cmd[100];
+  char *p;
+
+  value = _gpgme_data_get_size_hint (data);
+  if (!value)
+    value = 0;
+
+  p = numbuf + sizeof numbuf;
+  *--p = 0;
+  do
+    {
+      *--p = '0' + (value % 10);
+      value /= 10;
+    }
+  while (value);
+
+  snprintf (cmd, sizeof cmd, "OPTION input-size-hint=%s", p);
+  err = gpgsm_assuan_simple_command (gpgsm, cmd, NULL, NULL);
+  if (gpg_err_code (err) == GPG_ERR_UNKNOWN_OPTION)
+    err = 0; /* Ignore error from older gpgsm versions.  */
+  return err;
+}
+
 
 static gpgme_error_t
 gpgsm_decrypt (void *engine,
@@ -1258,6 +1364,14 @@ gpgsm_decrypt (void *engine,
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
+  err = send_input_size_hint (gpgsm, ciph);
+  if (err)
+    return err;
+
   gpgsm->input_cb.data = ciph;
   err = gpgsm_set_fd (gpgsm, INPUT_FD, map_data_enc (gpgsm->input_cb.data));
   if (err)
@@ -1269,7 +1383,7 @@ gpgsm_decrypt (void *engine,
   gpgsm_clear_fd (gpgsm, MESSAGE_FD);
   gpgsm->inline_data = NULL;
 
-  err = start (engine, "DECRYPT");
+  err = start (gpgsm, "DECRYPT");
   return err;
 }
 
@@ -1332,12 +1446,17 @@ gpgsm_delete (void *engine, gpgme_key_t key, unsigned int flags)
     }
   *linep = '\0';
 
+  err = prepare (gpgsm);
+  if (err)
+    goto leave;
+
   gpgsm_clear_fd (gpgsm, OUTPUT_FD);
   gpgsm_clear_fd (gpgsm, INPUT_FD);
   gpgsm_clear_fd (gpgsm, MESSAGE_FD);
   gpgsm->inline_data = NULL;
 
   err = start (gpgsm, line);
+ leave:
   free (line);
 
   return err;
@@ -1409,11 +1528,12 @@ set_recipients_from_string (engine_gpgsm_t gpgsm, const char *string)
 {
   gpg_error_t err = 0;
   char *line = NULL;
-  int no_pubkey = 0;
+  int ignore = 0;
+  int any = 0;
   const char *s;
   int n;
 
-  for (;;)
+  do
     {
       while (*string == ' ' || *string == '\t')
         string++;
@@ -1428,25 +1548,32 @@ set_recipients_from_string (engine_gpgsm_t gpgsm, const char *string)
       while (n && (string[n-1] == ' ' || string[n-1] == '\t'))
         n--;
 
-      gpgrt_free (line);
-      if (gpgrt_asprintf (&line, "RECIPIENT %.*s", n, string) < 0)
+      if (!ignore && n == 2 && !memcmp (string, "--", 2))
+        ignore = 1;
+      else if (!ignore && n > 2 && !memcmp (string, "--", 2))
+        err = gpg_error (GPG_ERR_UNKNOWN_OPTION);
+      else if (n) /* Not empty - use it.  */
         {
-          err = gpg_error_from_syserror ();
-          break;
+          gpgrt_free (line);
+          if (gpgrt_asprintf (&line, "RECIPIENT %.*s", n, string) < 0)
+            err = gpg_error_from_syserror ();
+          else
+            {
+              err = gpgsm_assuan_simple_command (gpgsm, line, gpgsm->status.fnc,
+                                                 gpgsm->status.fnc_value);
+              if (!err)
+                any = 1;
+            }
         }
+
       string += n + !!s;
-
-      err = gpgsm_assuan_simple_command (gpgsm, line, gpgsm->status.fnc,
-					 gpgsm->status.fnc_value);
-
-      /* Fixme: Improve error reporting.  */
-      if (gpg_err_code (err) == GPG_ERR_NO_PUBKEY)
-	no_pubkey++;
-      else if (err)
-        break;
     }
+  while (!err);
+
+  if (!err && !any)
+    err = gpg_error (GPG_ERR_MISSING_KEY);
   gpgrt_free (line);
-  return err? err : no_pubkey? gpg_error (GPG_ERR_NO_PUBKEY) : 0;
+  return err;
 }
 
 
@@ -1460,9 +1587,16 @@ gpgsm_encrypt (void *engine, gpgme_key_t recp[], const char *recpstring,
 
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
-  if (!recp)
+  if (!recp && !recpstring) /* Symmetric only */
     return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
 
+  if (flags & (GPGME_ENCRYPT_ARCHIVE | GPGME_ENCRYPT_FILE))
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
   if ((flags & GPGME_ENCRYPT_NO_ENCRYPT_TO))
     {
       err = gpgsm_assuan_simple_command (gpgsm,
@@ -1471,6 +1605,21 @@ gpgsm_encrypt (void *engine, gpgme_key_t recp[], const char *recpstring,
 	return err;
     }
 
+  if ((flags & GPGME_ENCRYPT_ALWAYS_TRUST))
+    {
+      /* Note that a RESET and the actual operation resets the
+       * always-trust option.  To support older gnupg versions we
+       * ignore the unknown option error.  */
+      err = gpgsm_assuan_simple_command (gpgsm,
+                                         "OPTION always-trust", NULL, NULL);
+      if (err && gpg_err_code (err) != GPG_ERR_UNKNOWN_OPTION)
+        return err;
+    }
+
+  err = send_input_size_hint (gpgsm, plain);
+  if (err)
+    return err;
+
   gpgsm->input_cb.data = plain;
   err = gpgsm_set_fd (gpgsm, INPUT_FD, map_data_enc (gpgsm->input_cb.data));
   if (err)
@@ -1506,6 +1655,12 @@ gpgsm_export (void *engine, const char *pattern, gpgme_export_mode_t mode,
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  if ((mode & ~(GPGME_EXPORT_MODE_SECRET
+                |GPGME_EXPORT_MODE_MINIMAL
+                |GPGME_EXPORT_MODE_RAW
+                |GPGME_EXPORT_MODE_PKCS12)))
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   if (!pattern)
     pattern = "";
 
@@ -1524,6 +1679,10 @@ gpgsm_export (void *engine, const char *pattern, gpgme_export_mode_t mode,
     }
   strcat (cmd, pattern);
 
+  err = prepare (gpgsm);
+  if (err)
+    goto leave;
+
   gpgsm->output_cb.data = keydata;
   err = gpgsm_set_fd (gpgsm, OUTPUT_FD, use_armor ? "--armor"
 		      : map_data_enc (gpgsm->output_cb.data));
@@ -1534,6 +1693,7 @@ gpgsm_export (void *engine, const char *pattern, gpgme_export_mode_t mode,
   gpgsm->inline_data = NULL;
 
   err = start (gpgsm, cmd);
+ leave:
   free (cmd);
   return err;
 }
@@ -1553,6 +1713,12 @@ gpgsm_export_ext (void *engine, const char *pattern[], gpgme_export_mode_t mode,
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  if ((mode & ~(GPGME_EXPORT_MODE_SECRET
+                |GPGME_EXPORT_MODE_MINIMAL
+                |GPGME_EXPORT_MODE_RAW
+                |GPGME_EXPORT_MODE_PKCS12)))
+    return gpg_error (GPG_ERR_NOT_SUPPORTED);
+
   if (pattern && *pattern)
     {
       const char **pat = pattern;
@@ -1625,6 +1791,10 @@ gpgsm_export_ext (void *engine, const char *pattern[], gpgme_export_mode_t mode,
     }
   *linep = '\0';
 
+  err = prepare (gpgsm);
+  if (err)
+    goto leave;
+
   gpgsm->output_cb.data = keydata;
   err = gpgsm_set_fd (gpgsm, OUTPUT_FD, use_armor ? "--armor"
 		      : map_data_enc (gpgsm->output_cb.data));
@@ -1635,6 +1805,7 @@ gpgsm_export_ext (void *engine, const char *pattern[], gpgme_export_mode_t mode,
   gpgsm->inline_data = NULL;
 
   err = start (gpgsm, line);
+ leave:
   free (line);
   return err;
 }
@@ -1661,6 +1832,10 @@ gpgsm_genkey (void *engine,
       if (!pubkey || seckey)
         return gpg_error (GPG_ERR_INV_VALUE);
 
+      err = prepare (gpgsm);
+      if (err)
+	return err;
+
       gpgsm->input_cb.data = help_data;
       err = gpgsm_set_fd (gpgsm, INPUT_FD, map_data_enc (gpgsm->input_cb.data));
       if (err)
@@ -1690,19 +1865,32 @@ gpgsm_genkey (void *engine,
 
 
 static gpgme_error_t
-gpgsm_import (void *engine, gpgme_data_t keydata, gpgme_key_t *keyarray)
+gpgsm_import (void *engine, gpgme_data_t keydata, gpgme_key_t *keyarray,
+              const char *keyids[], const char *import_filter,
+              const char *import_options, const char *key_origin)
 {
   engine_gpgsm_t gpgsm = engine;
   gpgme_error_t err;
   gpgme_data_encoding_t dataenc;
   int idx;
 
+  (void)import_filter;
+  (void)import_options;
+  (void)key_origin;
+
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  if (keyids)
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
   if (keydata && keyarray)
     return gpg_error (GPG_ERR_INV_VALUE); /* Only one is allowed.  */
 
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
   dataenc = gpgme_data_get_encoding (keydata);
 
   if (keyarray)
@@ -1791,7 +1979,7 @@ gpgsm_import (void *engine, gpgme_data_t keydata, gpgme_key_t *keyarray)
 
 static gpgme_error_t
 gpgsm_keylist (void *engine, const char *pattern, int secret_only,
-	       gpgme_keylist_mode_t mode, int engine_flags)
+	       gpgme_keylist_mode_t mode)
 {
   engine_gpgsm_t gpgsm = engine;
   char *line;
@@ -1806,6 +1994,10 @@ gpgsm_keylist (void *engine, const char *pattern, int secret_only,
   if (!pattern)
     pattern = "";
 
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
   /* Hack to make sure that the agent is started.  Only if the agent
      has been started an application may connect to the agent via
      GPGME_PROTOCOL_ASSUAN - for example to look for smartcards.  We
@@ -1847,12 +2039,6 @@ gpgsm_keylist (void *engine, const char *pattern, int secret_only,
                                "OPTION with-secret=1":
                                "OPTION with-secret=0" ,
                                NULL, NULL);
-  gpgsm_assuan_simple_command (gpgsm,
-                               (engine_flags & GPGME_ENGINE_FLAG_OFFLINE)?
-                               "OPTION offline=1":
-                               "OPTION offline=0" ,
-                               NULL, NULL);
-
 
   /* Length is "LISTSECRETKEYS " + p + '\0'.  */
   line = malloc (15 + strlen (pattern) + 1);
@@ -1882,7 +2068,7 @@ gpgsm_keylist (void *engine, const char *pattern, int secret_only,
 
 static gpgme_error_t
 gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
-		   int reserved, gpgme_keylist_mode_t mode, int engine_flags)
+		   int reserved, gpgme_keylist_mode_t mode)
 {
   engine_gpgsm_t gpgsm = engine;
   char *line;
@@ -1901,6 +2087,10 @@ gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
   if (mode & GPGME_KEYLIST_MODE_EXTERN)
     list_mode |= 2;
 
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
   /* Always send list-mode option because RESET does not reset it.  */
   if (gpgrt_asprintf (&line, "OPTION list-mode=%d", (list_mode & 3)) < 0)
     return gpg_error_from_syserror ();
@@ -1922,11 +2112,6 @@ gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
                                "OPTION with-secret=1":
                                "OPTION with-secret=0" ,
                                NULL, NULL);
-  gpgsm_assuan_simple_command (gpgsm,
-                               (engine_flags & GPGME_ENGINE_FLAG_OFFLINE)?
-                               "OPTION offline=1":
-                               "OPTION offline=0" ,
-                               NULL, NULL);
 
   if (pattern && *pattern)
     {
@@ -2014,7 +2199,7 @@ gpgsm_keylist_ext (void *engine, const char *pattern[], int secret_only,
 
 static gpgme_error_t
 gpgsm_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
-	    gpgme_sig_mode_t mode, int use_armor, int use_textmode,
+	    gpgme_sig_mode_t flags, int use_armor, int use_textmode,
 	    int include_certs, gpgme_ctx_t ctx /* FIXME */)
 {
   engine_gpgsm_t gpgsm = engine;
@@ -2028,6 +2213,15 @@ gpgsm_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  if (flags & (GPGME_SIG_MODE_CLEAR
+               | GPGME_SIG_MODE_ARCHIVE
+               | GPGME_SIG_MODE_FILE))
+    return gpg_error (GPG_ERR_INV_VALUE);
+
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
   /* FIXME: This does not work as RESET does not reset it so we can't
      revert back to default.  */
   if (include_certs != GPGME_INCLUDE_CERTS_DEFAULT)
@@ -2064,6 +2258,10 @@ gpgsm_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
         return err;
     }
 
+  err = send_input_size_hint (gpgsm, in);
+  if (err)
+    return err;
+
   gpgsm->input_cb.data = in;
   err = gpgsm_set_fd (gpgsm, INPUT_FD, map_data_enc (gpgsm->input_cb.data));
   if (err)
@@ -2076,15 +2274,16 @@ gpgsm_sign (void *engine, gpgme_data_t in, gpgme_data_t out,
   gpgsm_clear_fd (gpgsm, MESSAGE_FD);
   gpgsm->inline_data = NULL;
 
-  err = start (gpgsm, mode == GPGME_SIG_MODE_DETACH
+  err = start (gpgsm, (flags & GPGME_SIG_MODE_DETACH)
 	       ? "SIGN --detached" : "SIGN");
   return err;
 }
 
 
 static gpgme_error_t
-gpgsm_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
-	      gpgme_data_t plaintext, gpgme_ctx_t ctx)
+gpgsm_verify (void *engine, gpgme_verify_flags_t flags, gpgme_data_t sig,
+              gpgme_data_t signed_text, gpgme_data_t plaintext,
+              gpgme_ctx_t ctx)
 {
   engine_gpgsm_t gpgsm = engine;
   gpgme_error_t err;
@@ -2094,6 +2293,13 @@ gpgsm_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
   if (!gpgsm)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  if (flags & GPGME_VERIFY_ARCHIVE)
+    return gpg_error (GPG_ERR_NOT_IMPLEMENTED);
+
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
   gpgsm->input_cb.data = sig;
   err = gpgsm_set_fd (gpgsm, INPUT_FD, map_data_enc (gpgsm->input_cb.data));
   if (err)
@@ -2101,6 +2307,10 @@ gpgsm_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
   if (!signed_text)
     {
       /* Normal or cleartext signature.  */
+      err = send_input_size_hint (gpgsm, sig);
+      if (err)
+        return err;
+
       if (plaintext)
         {
           gpgsm->output_cb.data = plaintext;
@@ -2116,6 +2326,10 @@ gpgsm_verify (void *engine, gpgme_data_t sig, gpgme_data_t signed_text,
   else
     {
       /* Detached signature.  */
+      err = send_input_size_hint (gpgsm, signed_text);
+      if (err)
+        return err;
+
       gpgsm->message_cb.data = signed_text;
       err = gpgsm_set_fd (gpgsm, MESSAGE_FD, 0);
       gpgsm_clear_fd (gpgsm, OUTPUT_FD);
@@ -2171,6 +2385,10 @@ gpgsm_getauditlog (void *engine, gpgme_data_t output, unsigned int flags)
   if (!gpgsm->assuan_ctx)
     return gpg_error (GPG_ERR_INV_VALUE);
 
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
 #if USE_DESCRIPTOR_PASSING
   gpgsm->output_cb.data = output;
   err = gpgsm_set_fd (gpgsm, OUTPUT_FD, 0);
@@ -2264,6 +2482,10 @@ gpgsm_passwd (void *engine, gpgme_key_t key, unsigned int flags)
   if (!key || !key->subkeys || !key->subkeys->fpr)
     return gpg_error (GPG_ERR_INV_CERT_OBJ);
 
+  err = prepare (gpgsm);
+  if (err)
+    return err;
+
   if (gpgrt_asprintf (&line, "PASSWD -- %s", key->subkeys->fpr) < 0)
     return gpg_error_from_syserror ();
 
@@ -2316,12 +2538,15 @@ struct engine_ops _gpgme_engine_ops_gpgsm =
     gpgsm_keylist_ext,
     NULL,               /* keylist_data */
     NULL,               /* keysign */
+    NULL,               /* revsig */
     NULL,               /* tofu_policy */
     gpgsm_sign,
-    NULL,		/* trustlist */
     gpgsm_verify,
     gpgsm_getauditlog,
+    NULL,               /* setexpire */
+    NULL,               /* setownertrust */
     NULL,               /* opassuan_transact */
+    NULL,               /* getdirect */
     NULL,		/* conf_load */
     NULL,		/* conf_save */
     NULL,		/* conf_dir */
