@@ -54,11 +54,18 @@
 # include <dirent.h>
 #endif /*USE_LINUX_GETDENTS*/
 
+#ifdef HAVE_POLL_H
+# include <poll.h>
+#else
+# ifdef HAVE_SYS_SELECT_H
+#  include <sys/select.h>
+# endif
+#endif
+#include <sys/socket.h>
 
 #include "util.h"
 #include "priv-io.h"
 #include "sema.h"
-#include "ath.h"
 #include "debug.h"
 
 
@@ -127,6 +134,10 @@ _gpgme_is_fd_valid (int fd)
 #endif /*0*/
 
 
+/* Fallback value for max_fds for some systems.  It can be used later
+   in get_max_fds function.  */
+static int max_fds_fallback;
+
 void
 _gpgme_io_subsystem_init (void)
 {
@@ -140,6 +151,23 @@ _gpgme_io_subsystem_init (void)
       act.sa_flags = 0;
       sigaction (SIGPIPE, &act, NULL);
     }
+
+  max_fds_fallback = -1;
+
+#ifdef _SC_OPEN_MAX
+  if (max_fds_fallback == -1)
+    {
+      /* For multithread application, sysconf cannot be used in the
+       * forked child process (in get_max_fds function), since it is
+       * not async-signal-safe function.  Here, we can assume it's
+       * only a single thread.
+       */
+      long scres = sysconf (_SC_OPEN_MAX);
+
+      if (scres >= 0)
+        max_fds_fallback = scres;
+    }
+#endif
 }
 
 
@@ -178,7 +206,7 @@ _gpgme_io_read (int fd, void *buffer, size_t count)
 
   do
     {
-      nread = _gpgme_ath_read (fd, buffer, count);
+      nread = read (fd, buffer, count);
     }
   while (nread == -1 && errno == EINTR);
 
@@ -197,7 +225,7 @@ _gpgme_io_write (int fd, const void *buffer, size_t count)
 
   do
     {
-      nwritten = _gpgme_ath_write (fd, buffer, count);
+      nwritten = write (fd, buffer, count);
     }
   while (nwritten == -1 && errno == EINTR);
 
@@ -347,7 +375,7 @@ _gpgme_io_set_nonblocking (int fd)
 }
 
 
-static long int
+static int
 get_max_fds (void)
 {
   const char *source = NULL;
@@ -366,9 +394,11 @@ get_max_fds (void)
 #ifdef USE_LINUX_GETDENTS
   {
     int dir_fd;
-    char dir_buf[DIR_BUF_SIZE];
+    struct linux_dirent64 buf[(DIR_BUF_SIZE+sizeof (struct linux_dirent64)-1)
+                              /sizeof (struct linux_dirent64)];
+    char *dir_buf = (char *)buf; /* BUF aligned for struct linux_dirent64 */
     struct linux_dirent64 *dir_entry;
-    int r, pos;
+    long r, pos;
     const char *s;
     int x;
 
@@ -377,7 +407,7 @@ get_max_fds (void)
       {
         for (;;)
           {
-            r = syscall(SYS_getdents64, dir_fd, dir_buf, DIR_BUF_SIZE);
+            r = syscall(SYS_getdents64, dir_fd, dir_buf, sizeof (buf));
             if (r == -1)
               {
                 /* Fall back to other methods.  */
@@ -435,18 +465,13 @@ get_max_fds (void)
 	}
     }
 #endif
-#ifdef _SC_OPEN_MAX
-  if (fds == -1)
+
+  if (fds == -1 && max_fds_fallback >= 0)
     {
-      long int scres;
-      scres = sysconf (_SC_OPEN_MAX);
-      if (scres >= 0)
-	{
-	  source = "_SC_OPEN_MAX";
-	  return scres;
-	}
+      source = "fallback";
+      return max_fds_fallback;
     }
-#endif
+
 #ifdef OPEN_MAX
   if (fds == -1)
     {
@@ -476,7 +501,11 @@ get_max_fds (void)
     }
 #endif
 
+#if 0 /* This may result hang in multi-thread application.  */
   TRACE (DEBUG_SYSIO, "gpgme:max_fds", NULL, "max fds=%ld (%s)", fds, source);
+#else
+  (void)source;
+#endif
   return fds;
 }
 
@@ -490,7 +519,7 @@ _gpgme_io_waitpid (int pid, int hang, int *r_status, int *r_signal)
   *r_status = 0;
   *r_signal = 0;
   do
-    ret = _gpgme_ath_waitpid (pid, &status, hang? 0 : WNOHANG);
+    ret = waitpid (pid, &status, hang? 0 : WNOHANG);
   while (ret == (pid_t)(-1) && errno == EINTR);
 
   if (ret == pid)
@@ -770,9 +799,10 @@ _gpgme_io_select_poll (struct io_select_fd_s *fds, size_t nfds, int nonblock)
           if (fds[i].fd == -1)
             continue;
 	  if ((poll_fds[poll_nfds].revents & (POLLIN|POLLHUP)))
-	    TRACE_ADD1 (dbg_help, "r=%d ", i);
+	    TRACE_ADD2 (dbg_help, "r=%d (%x) ", fds[i].fd,
+                        (int)poll_fds[poll_nfds].revents);
 	  if ((poll_fds[poll_nfds].revents & POLLOUT))
-	    TRACE_ADD1 (dbg_help, "w=%d ", i);
+	    TRACE_ADD1 (dbg_help, "w=%d ", fds[i].fd);
           poll_nfds++;
         }
       TRACE_END (dbg_help, "]");
@@ -869,8 +899,7 @@ _gpgme_io_select_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
 
   do
     {
-      count = _gpgme_ath_select (max_fd + 1, &readfds, &writefds, NULL,
-				 &timeout);
+      count = select (max_fd + 1, &readfds, &writefds, NULL, &timeout);
     }
   while (count < 0 && errno == EINTR);
   if (count < 0)
@@ -946,7 +975,7 @@ _gpgme_io_recvmsg (int fd, struct msghdr *msg, int flags)
 
   do
     {
-      nread = _gpgme_ath_recvmsg (fd, msg, flags);
+      nread = recvmsg (fd, msg, flags);
     }
   while (nread == -1 && errno == EINTR);
   saved_errno = errno;
@@ -996,7 +1025,7 @@ _gpgme_io_sendmsg (int fd, const struct msghdr *msg, int flags)
 
   do
     {
-      nwritten = _gpgme_ath_sendmsg (fd, msg, flags);
+      nwritten = sendmsg (fd, msg, flags);
     }
   while (nwritten == -1 && errno == EINTR);
   return TRACE_SYSRES (nwritten);
@@ -1041,7 +1070,7 @@ _gpgme_io_connect (int fd, struct sockaddr *addr, int addrlen)
 	      "fd=%d addr=%p addrlen=%i", fd, addr, addrlen);
 
   do
-    res = ath_connect (fd, addr, addrlen);
+    res = connect (fd, addr, addrlen);
   while (res == -1 && errno == EINTR);
 
   return TRACE_SYSRES (res);
