@@ -1,23 +1,23 @@
-/* wait-global.c 
-   Copyright (C) 2000 Werner Koch (dd9jn)
-   Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
- 
-   This file is part of GPGME.
- 
-   GPGME is free software; you can redistribute it and/or modify it
-   under the terms of the GNU Lesser General Public License as
-   published by the Free Software Foundation; either version 2.1 of
-   the License, or (at your option) any later version.
-   
-   GPGME is distributed in the hope that it will be useful, but
-   WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-   Lesser General Public License for more details.
-   
-   You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+/* wait-global.c
+ * Copyright (C) 2000 Werner Koch (dd9jn)
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005 g10 Code GmbH
+ *
+ * This file is part of GPGME.
+ *
+ * GPGME is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as
+ * published by the Free Software Foundation; either version 2.1 of
+ * the License, or (at your option) any later version.
+ *
+ * GPGME is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ */
 
 #if HAVE_CONFIG_H
 #include <config.h>
@@ -34,6 +34,7 @@
 #include "wait.h"
 #include "priv-io.h"
 #include "ops.h"
+#include "debug.h"
 
 /* The global event loop is used for all asynchronous operations
    (except key listing) for which no user I/O callbacks are specified.
@@ -96,7 +97,7 @@ ctx_active (gpgme_ctx_t ctx)
 {
   struct ctx_list_item *li = malloc (sizeof (struct ctx_list_item));
   if (!li)
-    return gpg_error_from_errno (errno);
+    return gpg_error_from_syserror ();
   li->ctx = ctx;
 
   LOCK (ctx_list_lock);
@@ -205,7 +206,7 @@ _gpgme_wait_global_event_cb (void *data, gpgme_event_io_t type,
 	gpgme_error_t err = ctx_active (ctx);
 
 	if (err)
-	  /* An error occured.  Close all fds in this context, and
+	  /* An error occurred.  Close all fds in this context, and
 	     send the error in a done event.  */
 	  _gpgme_cancel_with_err (ctx, err, 0);
       }
@@ -224,10 +225,6 @@ _gpgme_wait_global_event_cb (void *data, gpgme_event_io_t type,
       assert (!"Unexpected event GPGME_EVENT_NEXT_KEY");
       break;
 
-    case GPGME_EVENT_NEXT_TRUSTITEM:
-      assert (!"Unexpected event GPGME_EVENT_NEXT_TRUSTITEM");
-      break;
-
     default:
       assert (!"Unexpected event");
       break;
@@ -268,10 +265,10 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
       fdt.fds = malloc (i * sizeof (struct io_select_fd_s));
       if (!fdt.fds)
 	{
-	  int saved_errno = errno;
+          int saved_err = gpg_error_from_syserror ();
 	  UNLOCK (ctx_list_lock);
 	  if (status)
-	    *status = gpg_error_from_errno (saved_errno);
+	    *status = saved_err;
 	  if (op_err)
 	    *op_err = 0;
 	  return NULL;
@@ -289,10 +286,10 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
       nr = _gpgme_io_select (fdt.fds, fdt.size, 0);
       if (nr < 0)
 	{
-	  int saved_errno = errno;
+          int saved_err = gpg_error_from_syserror ();
 	  free (fdt.fds);
 	  if (status)
-	    *status = gpg_error_from_errno (saved_errno);
+	    *status = saved_err;
 	  if (op_err)
 	    *op_err = 0;
 	  return NULL;
@@ -306,10 +303,10 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 	      gpgme_error_t err = 0;
 	      gpgme_error_t local_op_err = 0;
 	      struct wait_item_s *item;
-	      
+
 	      assert (nr);
 	      nr--;
-	      
+
 	      item = (struct wait_item_s *) fdt.fds[i].opaque;
 	      assert (item);
 	      ictx = item->ctx;
@@ -324,7 +321,7 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 		err = _gpgme_run_io_cb (&fdt.fds[i], 0, &local_op_err);
 	      if (err || local_op_err)
 		{
-		  /* An error occured.  Close all fds in this context,
+		  /* An error occurred.  Close all fds in this context,
 		     and signal it.  */
 		  _gpgme_cancel_with_err (ictx, err, local_op_err);
 
@@ -352,7 +349,7 @@ gpgme_wait_ext (gpgme_ctx_t ctx, gpgme_error_t *status,
 	      struct gpgme_io_event_done_data data;
 	      data.err = 0;
 	      data.op_err = 0;
-	      
+
 	      /* FIXME: This does not perform too well.  We have to
 		 release the lock because the I/O event handler
 		 acquires it to remove the context from the active
