@@ -15,7 +15,8 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this program; if not, see <https://gnu.org/licenses/>.
+ * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
 /* We need to include config.h so that we know whether we are building
@@ -35,9 +36,19 @@
 #include "t-support.h"
 
 
+static const char normal_signed_message[] =
+"-----BEGIN PGP MESSAGE-----\n"
+"\n"
+"owGbwMvMwCSoW1RzPCOz3IRxjXQSR0lqcYleSUWJTZOvjVdpcYmCu1+oQmaJIleH\n"
+"GwuDIBMDGysTSIqBi1MApi+nlGGuwDeHao53HBr+FoVGP3xX+kvuu9fCMJvl6IOf\n"
+"y1kvP4y+8D5a11ang0udywsA\n"
+"=Crq6\n"
+"-----END PGP MESSAGE-----\n";
+
+
 static void
 check_verify_result (gpgme_verify_result_t result, unsigned int summary,
-		     const char *fpr, gpgme_error_t status)
+		     const char *fpr, gpgme_error_t status, int validity)
 {
   gpgme_signature_t sig;
 
@@ -78,10 +89,11 @@ check_verify_result (gpgme_verify_result_t result, unsigned int summary,
 	       __FILE__, __LINE__);
       exit (1);
     }
-  if (sig->validity != GPGME_VALIDITY_UNKNOWN)
+  if (sig->validity != validity)
     {
-      fprintf (stderr, "%s:%i: Unexpected validity: %i\n",
-	       __FILE__, __LINE__, sig->validity);
+      fprintf (stderr, "%s:%i: Unexpected validity: "
+               "want=%i have=%i\n",
+	       __FILE__, __LINE__, validity, sig->validity);
       exit (1);
     }
   if (gpgme_err_code (sig->validity_reason) != GPG_ERR_NO_ERROR)
@@ -133,12 +145,33 @@ main (int argc, char *argv[])
     }
   print_data (out);
   verify_result = gpgme_op_verify_result (ctx);
-  check_verify_result (verify_result, 0,
+  check_verify_result (verify_result, GPGME_SIGSUM_VALID|GPGME_SIGSUM_GREEN,
 		       "A0FF4590BB6122EDEF6E3C542D727CC768697734",
-		       GPG_ERR_NO_ERROR);
+		       GPG_ERR_NO_ERROR, GPGME_VALIDITY_FULL);
 
   gpgme_data_release (in);
   gpgme_data_release (out);
+
+  /* Checking a signed, but not encrypted message.  */
+  err = gpgme_data_new_from_mem (&in, normal_signed_message, strlen (normal_signed_message), 0);
+  fail_if_err (err);
+  err = gpgme_data_new (&out);
+  fail_if_err (err);
+  err = gpgme_op_decrypt_verify (ctx, in, out);
+  /* should have returned "no data" because the message is not encrypted */
+  if (gpgme_err_code (err) != GPG_ERR_NO_DATA)
+    {
+      fprintf (stderr, "%s:%i: Unexpected result of gpgme_op_decrypt_verify: %s\n",
+	       __FILE__, __LINE__, gpgme_strerror (err));
+    }
+  verify_result = gpgme_op_verify_result (ctx);
+  check_verify_result (verify_result, GPGME_SIGSUM_VALID|GPGME_SIGSUM_GREEN,
+		       "A0FF4590BB6122EDEF6E3C542D727CC768697734",
+		       GPG_ERR_NO_ERROR, GPGME_VALIDITY_FULL);
+
+  gpgme_data_release (in);
+  gpgme_data_release (out);
+
   gpgme_release (ctx);
   return 0;
 }
