@@ -131,15 +131,23 @@ progress_cb (void *opaque, const char *what, int type, int current, int total)
 
 
 static unsigned long
-parse_expire_string (const char *string)
+parse_expire_string (const char *string, unsigned int *flags)
 {
-  unsigned long seconds;
+  unsigned long seconds = 0;
 
-  if (!string || !*string || !strcmp (string, "none")
-      || !strcmp (string, "never") || !strcmp (string, "-"))
-    seconds = 0;
+  if (!string || !*string || !strcmp (string, "-"))
+    ;
+  else if (!strcmp (string, "none") || !strcmp (string, "never"))
+    {
+      if (flags)
+        *flags |= GPGME_CREATE_NOEXPIRE;
+    }
   else if (strspn (string, "01234567890") == strlen (string))
-    seconds = strtoul (string, NULL, 10);
+    {
+      seconds = strtoul (string, NULL, 10);
+      if (!seconds && flags)
+        *flags |= GPGME_CREATE_NOEXPIRE;
+    }
   else
     {
       fprintf (stderr, PGM ": invalid value '%s'\n", string);
@@ -182,6 +190,8 @@ parse_usage_string (const char *string)
         flags |= GPGME_CREATE_CERT;
       else if (!strcmp (s, "auth"))
         flags |= GPGME_CREATE_AUTH;
+      else if (!strcmp (s, "group"))
+        flags |= GPGME_CREATE_GROUP;
       else
         {
           free (tokens);
@@ -205,12 +215,17 @@ show_usage (int ex)
          "   for addkey: FPR    [ALGO [USAGE [EXPIRESECONDS]]]\n"
          "   for adduid: FPR    USERID\n"
          "   for revuid: FPR    USERID\n"
+         "   for setexpire: FPR EXPIRE [SUBFPRS]\n"
          "   for set-primary: FPR    USERID\n"
+         "   for addadsk: FPR   ADSKFPR\n"
          "Options:\n"
          "  --addkey         add a subkey to the key with FPR\n"
          "  --adduid         add a user id to the key with FPR\n"
          "  --revuid         revoke a user id from the key with FPR\n"
          "  --set-primary    set the primary key flag on USERID\n"
+         "  --setexpire      set the expiration time of the key FPR\n"
+         "                   or of its subkeys SUBFPRS\n"
+         "  --addadsk        add the subkey with ADSKFPR to the key FPR\n"
          "  --verbose        run in verbose mode\n"
          "  --status         print status lines from the backend\n"
          "  --progress       print progress info\n"
@@ -238,12 +253,19 @@ main (int argc, char **argv)
   int adduid = 0;
   int revuid = 0;
   int setpri = 0;
+  int setexpire = 0;
+  int addadsk = 0;
   const char *userid;
   const char *algo = NULL;
   const char *newuserid = NULL;
+  const char *subfprs = NULL;
   unsigned int flags = 0;
   unsigned long expire = 0;
   gpgme_genkey_result_t result;
+  int i;
+  size_t n;
+  char *subfprs_buffer = NULL;
+  char *adskfpr = NULL;
 
   if (argc)
     { argc--; argv++; }
@@ -264,6 +286,8 @@ main (int argc, char **argv)
           adduid = 0;
           revuid = 0;
           setpri = 0;
+          setexpire = 0;
+          addadsk = 0;
           argc--; argv++;
         }
       else if (!strcmp (*argv, "--adduid"))
@@ -272,6 +296,8 @@ main (int argc, char **argv)
           adduid = 1;
           revuid = 0;
           setpri = 0;
+          setexpire = 0;
+          addadsk = 0;
           argc--; argv++;
         }
       else if (!strcmp (*argv, "--revuid"))
@@ -280,6 +306,8 @@ main (int argc, char **argv)
           adduid = 0;
           revuid = 1;
           setpri = 0;
+          setexpire = 0;
+          addadsk = 0;
           argc--; argv++;
         }
       else if (!strcmp (*argv, "--set-primary"))
@@ -288,6 +316,28 @@ main (int argc, char **argv)
           adduid = 0;
           revuid = 0;
           setpri = 1;
+          setexpire = 0;
+          addadsk = 0;
+          argc--; argv++;
+        }
+      else if (!strcmp (*argv, "--setexpire"))
+        {
+          addkey = 0;
+          adduid = 0;
+          revuid = 0;
+          setpri = 0;
+          setexpire = 1;
+          addadsk = 0;
+          argc--; argv++;
+        }
+      else if (!strcmp (*argv, "--addadsk"))
+        {
+          addkey = 0;
+          adduid = 0;
+          revuid = 0;
+          setpri = 0;
+          setexpire = 0;
+          addadsk = 1;
           argc--; argv++;
         }
       else if (!strcmp (*argv, "--verbose"))
@@ -341,6 +391,55 @@ main (int argc, char **argv)
       userid = argv[0];
       newuserid = argv[1];
     }
+  else if (setexpire)
+    {
+      if (argc < 2)
+        {
+          show_usage (1);
+        }
+      userid = argv[0];
+      argc--; argv++;
+      expire = parse_expire_string (argv[0], NULL);
+      argc--; argv++;
+      if (argc > 1)
+        {
+          /* Several subkey fprs given  */
+          for (i=0, n = 0; i < argc; i++)
+            n += strlen (argv[1]) + 1;
+          n++;
+          subfprs_buffer = malloc (n);
+          if (!subfprs_buffer)
+            {
+              fprintf (stderr, PGM ": malloc failed: %s\n",
+                       gpg_strerror (gpg_error_from_syserror ()));
+              exit (1);
+            }
+          *subfprs_buffer = 0;
+          for (i=0; i < argc; i++)
+            {
+              strcat (subfprs_buffer, argv[i]);
+              strcat (subfprs_buffer, "\n");
+            }
+          subfprs = subfprs_buffer;
+        }
+      else if (argc)
+        {
+          /* One subkey fpr (or '*') given  */
+          subfprs = *argv;
+        }
+      else
+        {
+          /* No subkey fpr given.  */
+          subfprs = NULL;
+        }
+    }
+  else if (addadsk)
+    {
+      if (argc != 2)
+        show_usage(1);
+      userid = argv[0];
+      adskfpr = argv[1];
+    }
   else
     {
       if (!argc || argc > 4)
@@ -351,7 +450,7 @@ main (int argc, char **argv)
       if (argc > 2)
         flags |= parse_usage_string (argv[2]);
       if (argc > 3)
-        expire = parse_expire_string (argv[3]);
+        expire = parse_expire_string (argv[3], &flags);
     }
 
   init_gpgme (protocol);
@@ -373,7 +472,8 @@ main (int argc, char **argv)
       gpgme_set_passphrase_cb (ctx, passphrase_cb, NULL);
     }
 
-  if (addkey || adduid || revuid || setpri)
+
+  if (addkey || adduid || revuid || setpri || setexpire || addadsk)
     {
       gpgme_key_t akey;
 
@@ -425,6 +525,28 @@ main (int argc, char **argv)
               exit (1);
             }
         }
+      else if (setexpire)
+        {
+          err = gpgme_op_setexpire (ctx, akey, expire, subfprs, 0);
+          if (err)
+            {
+              fprintf (stderr, PGM ": gpgme_op_setexpire failed: %s\n",
+                      gpg_strerror (err));
+              exit (1);
+            }
+        }
+      else if (addadsk)
+        {
+          err = gpgme_op_createsubkey(ctx, akey, adskfpr, 0, 0,
+                                      GPGME_CREATE_ADSK);
+          if (err)
+            {
+              fprintf (stderr, PGM ": gpgme_op_createsubkey failed: %s\n",
+                      gpg_strerror (err));
+              exit (1);
+            }
+        }
+
       gpgme_key_unref (akey);
     }
   else
@@ -438,7 +560,7 @@ main (int argc, char **argv)
         }
     }
 
-  if (!setpri)
+  if (!setpri && !setexpire && !addadsk)
     {
       result = gpgme_op_genkey_result (ctx);
       if (!result)
